@@ -33,6 +33,7 @@
 #include "qgsgeos.h"
 #include "qgsmessagelog.h"
 #include "qgsgeometryutils.h"
+#include "qgsgeometryutils_base.h"
 #include <qglobal.h>
 
 using namespace pal;
@@ -91,14 +92,14 @@ PointSet::PointSet( const PointSet &ps )
 
   if ( ps.mGeos )
   {
-    mGeos = GEOSGeom_clone_r( QgsGeos::getGEOSHandler(), ps.mGeos );
+    mGeos = GEOSGeom_clone_r( QgsGeosContext::get(), ps.mGeos );
     mOwnsGeom = true;
   }
 }
 
 void PointSet::createGeosGeom() const
 {
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
 
   bool needClose = false;
   if ( type == GEOS_POLYGON && ( !qgsDoubleNear( x[0], x[ nbPoints - 1] ) || !qgsDoubleNear( y[0], y[ nbPoints - 1 ] ) ) )
@@ -158,14 +159,14 @@ const GEOSPreparedGeometry *PointSet::preparedGeom() const
 
   if ( !mPreparedGeom )
   {
-    mPreparedGeom = GEOSPrepare_r( QgsGeos::getGEOSHandler(), mGeos );
+    mPreparedGeom = GEOSPrepare_r( QgsGeosContext::get(), mGeos );
   }
   return mPreparedGeom;
 }
 
 void PointSet::invalidateGeos() const
 {
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
   if ( mOwnsGeom ) // delete old geometry if we own it
     GEOSGeom_destroy_r( geosctxt, mGeos );
   mOwnsGeom = false;
@@ -200,7 +201,7 @@ void PointSet::invalidateGeos() const
 
 PointSet::~PointSet()
 {
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
 
   if ( mGeos && mOwnsGeom )
   {
@@ -239,7 +240,7 @@ std::unique_ptr<PointSet> PointSet::extractShape( int nbPtSh, int imin, int imax
 {
   int i, j;
 
-  std::unique_ptr<PointSet> newShape = std::make_unique< PointSet >();
+  auto newShape = std::make_unique< PointSet >();
   newShape->type = GEOS_POLYGON;
   newShape->nbPoints = nbPtSh;
   newShape->x.resize( newShape->nbPoints );
@@ -269,7 +270,7 @@ std::unique_ptr<PointSet> PointSet::clone() const
 
 bool PointSet::containsPoint( double x, double y ) const
 {
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
   try
   {
     geos::unique_ptr point( GEOSGeom_createPointFromXY_r( geosctxt, x, y ) );
@@ -357,13 +358,13 @@ QLinkedList<PointSet *> PointSet::splitPolygons( PointSet *inputShape, double la
         if ( pt  != -1 )
         {
           // compute the ihs->ihn->pt triangle's area
-          const double base = GeomFunction::dist_euc2d( x[shape->convexHull[ihs]], y[shape->convexHull[ihs]],
+          const double base = QgsGeometryUtilsBase::distance2D( x[shape->convexHull[ihs]], y[shape->convexHull[ihs]],
                               x[shape->convexHull[ihn]], y[shape->convexHull[ihn]] );
 
-          b = GeomFunction::dist_euc2d( x[shape->convexHull[ihs]], y[shape->convexHull[ihs]],
-                                        x[pt], y[pt] );
+          b = QgsGeometryUtilsBase::distance2D( x[shape->convexHull[ihs]], y[shape->convexHull[ihs]],
+                                                x[pt], y[pt] );
 
-          const double c = GeomFunction::dist_euc2d( x[shape->convexHull[ihn]], y[shape->convexHull[ihn]],
+          const double c = QgsGeometryUtilsBase::distance2D( x[shape->convexHull[ihn]], y[shape->convexHull[ihn]],
                            x[pt], y[pt] );
 
           const double s = ( base + b + c ) / 2; // s = half perimeter
@@ -406,7 +407,7 @@ QLinkedList<PointSet *> PointSet::splitPolygons( PointSet *inputShape, double la
         // compute distance between retainedPoint and segment
         // whether perpendicular distance (if retaindPoint is fronting segment i->j)
         // or distance between retainedPt and i or j (choose the nearest)
-        seg_length = GeomFunction::dist_euc2d( x[i], y[i], x[j], y[j] );
+        seg_length = QgsGeometryUtilsBase::distance2D( x[i], y[i], x[j], y[j] );
         cx = ( x[i] + x[j] ) / 2.0;
         cy = ( y[i] + y[j] ) / 2.0;
         dx = cy - y[i];
@@ -419,7 +420,7 @@ QLinkedList<PointSet *> PointSet::splitPolygons( PointSet *inputShape, double la
 
         if ( seg_length < EPSILON || std::fabs( ( b = GeomFunction::cross_product( ex, ey, fx, fy, x[retainedPt], y[retainedPt] ) / ( seg_length ) ) ) > ( seg_length / 2 ) )   // retainedPt is not fronting i->j
         {
-          if ( ( ex = GeomFunction::dist_euc2d_sq( x[i], y[i], x[retainedPt], y[retainedPt] ) ) < ( ey = GeomFunction::dist_euc2d_sq( x[j], y[j], x[retainedPt], y[retainedPt] ) ) )
+          if ( ( ex = QgsGeometryUtilsBase::sqrDistance2D( x[i], y[i], x[retainedPt], y[retainedPt] ) ) < ( ey = QgsGeometryUtilsBase::sqrDistance2D( x[j], y[j], x[retainedPt], y[retainedPt] ) ) )
           {
             b = ex;
             ps = i;
@@ -550,7 +551,7 @@ void PointSet::offsetCurveByDistance( double distance )
   if ( !mGeos || type != GEOS_LINESTRING )
     return;
 
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
   geos::unique_ptr newGeos = nullptr;
   try
   {
@@ -648,7 +649,7 @@ void PointSet::extendLineByDistance( double startDistance, double endDistance, d
     {
       const double thisX = x[i];
       const double thisY = y[i];
-      const double thisSegmentLength = std::sqrt( ( thisX - lastX ) * ( thisX - lastX ) + ( thisY - lastY ) * ( thisY - lastY ) );
+      const double thisSegmentLength = QgsGeometryUtilsBase::distance2D( thisX, thisY, lastX, lastY );
       distanceConsumed += thisSegmentLength;
       if ( distanceConsumed >= smoothDistance )
       {
@@ -661,7 +662,7 @@ void PointSet::extendLineByDistance( double startDistance, double endDistance, d
       lastY = thisY;
     }
 
-    const double distance = std::sqrt( ( x1 - x0 ) * ( x1 - x0 ) + ( y1 - y0 ) * ( y1 - y0 ) );
+    const double distance = QgsGeometryUtilsBase::distance2D( x1, y1, x0, y0 );
     const double extensionFactor = ( startDistance + distance ) / distance;
     const QgsPointXY newStart = QgsGeometryUtils::interpolatePointOnLine( x1, y1, x0, y0, extensionFactor );
     x0 = newStart.x();
@@ -684,7 +685,7 @@ void PointSet::extendLineByDistance( double startDistance, double endDistance, d
     {
       const double thisX = x[i];
       const double thisY = y[i];
-      const double thisSegmentLength = std::sqrt( ( thisX - lastX ) * ( thisX - lastX ) + ( thisY - lastY ) * ( thisY - lastY ) );
+      const double thisSegmentLength = QgsGeometryUtilsBase::distance2D( thisX, thisY, lastX, lastY );
       distanceConsumed += thisSegmentLength;
       if ( distanceConsumed >= smoothDistance )
       {
@@ -697,7 +698,7 @@ void PointSet::extendLineByDistance( double startDistance, double endDistance, d
       lastY = thisY;
     }
 
-    const double distance = std::sqrt( ( xend1 - xend0 ) * ( xend1 - xend0 ) + ( yend1 - yend0 ) * ( yend1 - yend0 ) );
+    const double distance = QgsGeometryUtilsBase::distance2D( xend1, yend1, xend0, yend0 );
     const double extensionFactor = ( endDistance + distance ) / distance;
     const QgsPointXY newEnd = QgsGeometryUtils::interpolatePointOnLine( xend1, yend1, xend0, yend0, extensionFactor );
     x.emplace_back( newEnd.x() );
@@ -865,7 +866,7 @@ double PointSet::minDistanceToPoint( double px, double py, double *rx, double *r
   if ( !mGeos )
     return 0;
 
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
   try
   {
     geos::unique_ptr geosPt( GEOSGeom_createPointFromXY_r( geosctxt, px, py ) );
@@ -904,7 +905,7 @@ double PointSet::minDistanceToPoint( double px, double py, double *rx, double *r
     if ( ry )
       *ry = ny;
 
-    return GeomFunction::dist_euc2d_sq( px, py, nx, ny );
+    return QgsGeometryUtilsBase::sqrDistance2D( px, py, nx, ny );
   }
   catch ( GEOSException &e )
   {
@@ -924,7 +925,7 @@ void PointSet::getCentroid( double &px, double &py, bool forceInside ) const
 
   try
   {
-    GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+    GEOSContextHandle_t geosctxt = QgsGeosContext::get();
     geos::unique_ptr centroidGeom( GEOSGetCentroid_r( geosctxt, mGeos ) );
     if ( centroidGeom )
     {
@@ -1019,7 +1020,7 @@ geos::unique_ptr PointSet::interpolatePoint( double distance ) const
 
   try
   {
-    geos::unique_ptr res( GEOSInterpolate_r( QgsGeos::getGEOSHandler(), thisGeos, distance ) );
+    geos::unique_ptr res( GEOSInterpolate_r( QgsGeosContext::get(), thisGeos, distance ) );
     return res;
   }
   catch ( GEOSException &e )
@@ -1038,7 +1039,7 @@ double PointSet::lineLocatePoint( const GEOSGeometry *point ) const
   double distance = -1;
   try
   {
-    distance = GEOSProject_r( QgsGeos::getGEOSHandler(), thisGeos, point );
+    distance = GEOSProject_r( QgsGeosContext::get(), thisGeos, point );
   }
   catch ( GEOSException &e )
   {
@@ -1068,7 +1069,7 @@ double PointSet::length() const
   if ( !mGeos )
     return -1;
 
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
 
   try
   {
@@ -1094,7 +1095,7 @@ double PointSet::area() const
   if ( !mGeos )
     return -1;
 
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
 
   try
   {
@@ -1120,7 +1121,7 @@ QString PointSet::toWkt() const
   if ( !mGeos )
     createGeosGeom();
 
-  GEOSContextHandle_t geosctxt = QgsGeos::getGEOSHandler();
+  GEOSContextHandle_t geosctxt = QgsGeosContext::get();
 
   try
   {
