@@ -61,6 +61,7 @@ email                : tim at linfiniti.com
 #include "qgsthreadingutils.h"
 #include "qgssettingsentryimpl.h"
 #include "qgssettingstree.h"
+#include "qgsrasterlabeling.h"
 
 #include <cmath>
 #include <cstdio>
@@ -100,12 +101,12 @@ QgsRasterLayer::MULTIPLE_BAND_SINGLE_BYTE_ENHANCEMENT_ALGORITHM = QgsContrastEnh
 const QgsContrastEnhancement::ContrastEnhancementAlgorithm
 QgsRasterLayer::MULTIPLE_BAND_MULTI_BYTE_ENHANCEMENT_ALGORITHM = QgsContrastEnhancement::StretchToMinimumMaximum;
 
-const QgsRasterMinMaxOrigin::Limits
-QgsRasterLayer::SINGLE_BAND_MIN_MAX_LIMITS = QgsRasterMinMaxOrigin::MinMax;
-const QgsRasterMinMaxOrigin::Limits
-QgsRasterLayer::MULTIPLE_BAND_SINGLE_BYTE_MIN_MAX_LIMITS = QgsRasterMinMaxOrigin::MinMax;
-const QgsRasterMinMaxOrigin::Limits
-QgsRasterLayer::MULTIPLE_BAND_MULTI_BYTE_MIN_MAX_LIMITS = QgsRasterMinMaxOrigin::CumulativeCut;
+const Qgis::RasterRangeLimit
+QgsRasterLayer::SINGLE_BAND_MIN_MAX_LIMITS = Qgis::RasterRangeLimit::MinimumMaximum;
+const Qgis::RasterRangeLimit
+QgsRasterLayer::MULTIPLE_BAND_SINGLE_BYTE_MIN_MAX_LIMITS = Qgis::RasterRangeLimit::MinimumMaximum;
+const Qgis::RasterRangeLimit
+QgsRasterLayer::MULTIPLE_BAND_MULTI_BYTE_MIN_MAX_LIMITS = Qgis::RasterRangeLimit::CumulativeCut;
 
 QgsRasterLayer::QgsRasterLayer()
   : QgsMapLayer( Qgis::LayerType::Raster )
@@ -156,6 +157,8 @@ QgsRasterLayer::~QgsRasterLayer()
 {
   emit willBeDeleted();
 
+  mLabeling.reset();
+
   setValid( false );
   // Note: provider and other interfaces are owned and deleted by pipe
 }
@@ -169,7 +172,7 @@ QgsRasterLayer *QgsRasterLayer::clone() const
   {
     options.transformContext = mDataProvider->transformContext();
   }
-  std::unique_ptr< QgsRasterLayer > layer = std::make_unique< QgsRasterLayer >( source(), name(), mProviderKey, options );
+  auto layer = std::make_unique< QgsRasterLayer >( source(), name(), mProviderKey, options );
   QgsMapLayer::clone( layer.get() );
   layer->mElevationProperties = mElevationProperties->clone();
   layer->mElevationProperties->setParent( layer.get() );
@@ -190,6 +193,10 @@ QgsRasterLayer *QgsRasterLayer::clone() const
     layer->dataProvider()->setZoomedOutResamplingMethod( mDataProvider->zoomedOutResamplingMethod() );
   }
 
+  layer->setLabelsEnabled( mLabelsEnabled );
+  if ( mLabeling )
+    layer->setLabeling( mLabeling->clone() );
+
   return layer.release();
 }
 
@@ -637,13 +644,6 @@ QPixmap QgsRasterLayer::paletteAsPixmap( int bandNumber )
   }
 }
 
-QString QgsRasterLayer::providerType() const
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  return mProviderKey;
-}
-
 double QgsRasterLayer::rasterUnitsPerPixelX() const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -851,7 +851,7 @@ void QgsRasterLayer::setDataProvider( QString const &provider, const QgsDataProv
     QgsDebugMsgLevel( "Native Raster Raster Attribute Table read failed " + errorMessage, 2 );
     if ( QFile::exists( mDataProvider->dataSourceUri( ) +  ".vat.dbf" ) )
     {
-      std::unique_ptr<QgsRasterAttributeTable> rat = std::make_unique<QgsRasterAttributeTable>();
+      auto rat = std::make_unique<QgsRasterAttributeTable>();
       hasRat = rat->readFromFile( mDataProvider->dataSourceUri( ) +  ".vat.dbf", &errorMessage );
       if ( hasRat )
       {
@@ -957,18 +957,18 @@ void QgsRasterLayer::setDataProvider( QString const &provider, const QgsDataProv
     if ( resampling == QLatin1String( "bilinear" ) )
     {
       resampleFilter->setZoomedInResampler( new QgsBilinearRasterResampler() );
-      mDataProvider->setZoomedInResamplingMethod( QgsRasterDataProvider::ResamplingMethod::Bilinear );
+      mDataProvider->setZoomedInResamplingMethod( Qgis::RasterResamplingMethod::Bilinear );
     }
     else if ( resampling == QLatin1String( "cubic" ) )
     {
       resampleFilter->setZoomedInResampler( new QgsCubicRasterResampler() );
-      mDataProvider->setZoomedInResamplingMethod( QgsRasterDataProvider::ResamplingMethod::Cubic );
+      mDataProvider->setZoomedInResamplingMethod( Qgis::RasterResamplingMethod::Cubic );
     }
     resampling = settings.value( QStringLiteral( "/Raster/defaultZoomedOutResampling" ), QStringLiteral( "nearest neighbour" ) ).toString();
     if ( resampling == QLatin1String( "bilinear" ) )
     {
       resampleFilter->setZoomedOutResampler( new QgsBilinearRasterResampler() );
-      mDataProvider->setZoomedOutResamplingMethod( QgsRasterDataProvider::ResamplingMethod::Bilinear );
+      mDataProvider->setZoomedOutResamplingMethod( Qgis::RasterResamplingMethod::Bilinear );
     }
 
     const double maxOversampling = QgsRasterLayer::settingsRasterDefaultOversampling->value();
@@ -1185,7 +1185,7 @@ void QgsRasterLayer::readRasterAttributeTableExternalPaths( const QDomNode &laye
           continue;
         }
 
-        std::unique_ptr<QgsRasterAttributeTable> rat = std::make_unique<QgsRasterAttributeTable>();
+        auto rat = std::make_unique<QgsRasterAttributeTable>();
         QString errorMessage;
         if ( ! rat->readFromFile( path, &errorMessage ) )
         {
@@ -1211,7 +1211,7 @@ void QgsRasterLayer::closeDataProvider()
 
 void QgsRasterLayer::computeMinMax( int band,
                                     const QgsRasterMinMaxOrigin &mmo,
-                                    QgsRasterMinMaxOrigin::Limits limits,
+                                    Qgis::RasterRangeLimit limits,
                                     const QgsRectangle &extent,
                                     int sampleSize,
                                     double &min, double &max )
@@ -1223,7 +1223,7 @@ void QgsRasterLayer::computeMinMax( int band,
   if ( !mDataProvider )
     return;
 
-  if ( limits == QgsRasterMinMaxOrigin::MinMax )
+  if ( limits == Qgis::RasterRangeLimit::MinimumMaximum )
   {
     QgsRasterBandStats myRasterBandStats = mDataProvider->bandStatistics( band, Qgis::RasterBandStatistic::Min | Qgis::RasterBandStatistic::Max, extent, sampleSize );
     // Check if statistics were actually gathered, None means a failure
@@ -1296,13 +1296,13 @@ void QgsRasterLayer::computeMinMax( int band,
     min = myRasterBandStats.minimumValue;
     max = myRasterBandStats.maximumValue;
   }
-  else if ( limits == QgsRasterMinMaxOrigin::StdDev )
+  else if ( limits == Qgis::RasterRangeLimit::StdDev )
   {
     const QgsRasterBandStats myRasterBandStats = mDataProvider->bandStatistics( band, Qgis::RasterBandStatistic::Mean | Qgis::RasterBandStatistic::StdDev, extent, sampleSize );
     min = myRasterBandStats.mean - ( mmo.stdDevFactor() * myRasterBandStats.stdDev );
     max = myRasterBandStats.mean + ( mmo.stdDevFactor() * myRasterBandStats.stdDev );
   }
-  else if ( limits == QgsRasterMinMaxOrigin::CumulativeCut )
+  else if ( limits == Qgis::RasterRangeLimit::CumulativeCut )
   {
     const double myLower = mmo.cumulativeCutLower();
     const double myUpper = mmo.cumulativeCutUpper();
@@ -1334,7 +1334,7 @@ QgsMapLayerElevationProperties *QgsRasterLayer::elevationProperties()
   return mElevationProperties;
 }
 
-void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnhancementAlgorithm algorithm, QgsRasterMinMaxOrigin::Limits limits, const QgsRectangle &extent, int sampleSize, bool generateLookupTableFlag )
+void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnhancementAlgorithm algorithm, Qgis::RasterRangeLimit limits, const QgsRectangle &extent, int sampleSize, bool generateLookupTableFlag )
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
@@ -1347,7 +1347,7 @@ void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnh
 }
 
 void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnhancementAlgorithm algorithm,
-    QgsRasterMinMaxOrigin::Limits limits,
+    Qgis::RasterRangeLimit limits,
     const QgsRectangle &extent,
     int sampleSize,
     bool generateLookupTableFlag,
@@ -1355,7 +1355,7 @@ void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnh
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
-  QgsDebugMsgLevel( QStringLiteral( "theAlgorithm = %1 limits = %2 extent.isEmpty() = %3" ).arg( algorithm ).arg( limits ).arg( extent.isEmpty() ), 4 );
+  QgsDebugMsgLevel( QStringLiteral( "theAlgorithm = %1 limits = %2 extent.isEmpty() = %3" ).arg( algorithm ).arg( qgsEnumValueToKey( limits ) ).arg( extent.isEmpty() ), 4 );
   if ( !rasterRenderer || !mDataProvider )
   {
     return;
@@ -1412,7 +1412,7 @@ void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnh
     if ( myBand != -1 )
     {
       const Qgis::DataType myType = static_cast< Qgis::DataType >( mDataProvider->dataType( myBand ) );
-      std::unique_ptr<QgsContrastEnhancement> myEnhancement( new QgsContrastEnhancement( static_cast< Qgis::DataType >( myType ) ) );
+      auto myEnhancement = std::make_unique<QgsContrastEnhancement>( static_cast< Qgis::DataType >( myType ) );
       myEnhancement->setContrastEnhancementAlgorithm( algorithm, generateLookupTableFlag );
 
       double min;
@@ -1482,9 +1482,9 @@ void QgsRasterLayer::setContrastEnhancement( QgsContrastEnhancement::ContrastEnh
 
   myMinMaxOrigin.setLimits( limits );
   if ( extent != QgsRectangle() &&
-       myMinMaxOrigin.extent() == QgsRasterMinMaxOrigin::WholeRaster )
+       myMinMaxOrigin.extent() == Qgis::RasterRangeExtent::WholeRaster )
   {
-    myMinMaxOrigin.setExtent( QgsRasterMinMaxOrigin::CurrentCanvas );
+    myMinMaxOrigin.setExtent( Qgis::RasterRangeExtent::FixedCanvas );
   }
   if ( myRasterRenderer )
   {
@@ -1519,8 +1519,8 @@ void QgsRasterLayer::refreshContrastEnhancement( const QgsRectangle &extent )
   {
     setContrastEnhancement( ce->contrastEnhancementAlgorithm() == QgsContrastEnhancement::NoEnhancement ?
                             QgsContrastEnhancement::StretchToMinimumMaximum : ce->contrastEnhancementAlgorithm(),
-                            renderer()->minMaxOrigin().limits() == QgsRasterMinMaxOrigin::None ?
-                            QgsRasterMinMaxOrigin::MinMax : renderer()->minMaxOrigin().limits(),
+                            renderer()->minMaxOrigin().limits() == Qgis::RasterRangeLimit::NotSet ?
+                            Qgis::RasterRangeLimit::MinimumMaximum : renderer()->minMaxOrigin().limits(),
                             extent,
                             static_cast<int>( SAMPLE_SIZE ),
                             true,
@@ -1529,7 +1529,7 @@ void QgsRasterLayer::refreshContrastEnhancement( const QgsRectangle &extent )
   else
   {
     QgsContrastEnhancement::ContrastEnhancementAlgorithm myAlgorithm;
-    QgsRasterMinMaxOrigin::Limits myLimits;
+    Qgis::RasterRangeLimit myLimits;
     if ( defaultContrastEnhancementSettings( myAlgorithm, myLimits ) )
     {
       setContrastEnhancement( QgsContrastEnhancement::StretchToMinimumMaximum,
@@ -1542,115 +1542,6 @@ void QgsRasterLayer::refreshContrastEnhancement( const QgsRectangle &extent )
   }
 }
 
-void QgsRasterLayer::refreshRendererIfNeeded( QgsRasterRenderer *rasterRenderer,
-    const QgsRectangle &extent )
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  if ( mDataProvider &&
-       mLastRectangleUsedByRefreshContrastEnhancementIfNeeded != extent &&
-       rasterRenderer->minMaxOrigin().limits() != QgsRasterMinMaxOrigin::None &&
-       rasterRenderer->minMaxOrigin().extent() == QgsRasterMinMaxOrigin::UpdatedCanvas )
-  {
-    refreshRenderer( rasterRenderer, extent );
-  }
-}
-
-void QgsRasterLayer::refreshRenderer( QgsRasterRenderer *rasterRenderer, const QgsRectangle &extent )
-{
-  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
-
-  if ( mDataProvider )
-  {
-    QgsSingleBandGrayRenderer *singleBandRenderer = nullptr;
-    QgsMultiBandColorRenderer *multiBandRenderer = nullptr;
-    QgsSingleBandPseudoColorRenderer *sbpcr = nullptr;
-    const QgsContrastEnhancement *ce = nullptr;
-    if ( ( singleBandRenderer = dynamic_cast<QgsSingleBandGrayRenderer *>( rasterRenderer ) ) )
-    {
-      ce = singleBandRenderer->contrastEnhancement();
-    }
-    else if ( ( multiBandRenderer = dynamic_cast<QgsMultiBandColorRenderer *>( rasterRenderer ) ) )
-    {
-      ce = multiBandRenderer->redContrastEnhancement();
-    }
-    else if ( ( sbpcr = dynamic_cast<QgsSingleBandPseudoColorRenderer *>( rasterRenderer ) ) )
-    {
-      mLastRectangleUsedByRefreshContrastEnhancementIfNeeded = extent;
-      double min;
-      double max;
-      computeMinMax( sbpcr->inputBand(),
-                     rasterRenderer->minMaxOrigin(),
-                     rasterRenderer->minMaxOrigin().limits(), extent,
-                     static_cast<int>( SAMPLE_SIZE ), min, max );
-      sbpcr->setClassificationMin( min );
-      sbpcr->setClassificationMax( max );
-
-      if ( sbpcr->shader() )
-      {
-        QgsColorRampShader *colorRampShader = dynamic_cast<QgsColorRampShader *>( sbpcr->shader()->rasterShaderFunction() );
-        if ( colorRampShader )
-        {
-          colorRampShader->classifyColorRamp( sbpcr->inputBand(), extent, rasterRenderer->input() );
-        }
-      }
-
-      QgsSingleBandPseudoColorRenderer *r = dynamic_cast<QgsSingleBandPseudoColorRenderer *>( renderer() );
-      r->setClassificationMin( min );
-      r->setClassificationMax( max );
-
-      if ( r->shader() )
-      {
-        QgsColorRampShader *colorRampShader = dynamic_cast<QgsColorRampShader *>( r->shader()->rasterShaderFunction() );
-        if ( colorRampShader )
-        {
-          colorRampShader->classifyColorRamp( sbpcr->inputBand(), extent, rasterRenderer->input() );
-        }
-      }
-
-      emit repaintRequested();
-      emitStyleChanged();
-      emit rendererChanged();
-      return;
-    }
-
-    if ( ce &&
-         ce->contrastEnhancementAlgorithm() != QgsContrastEnhancement::NoEnhancement )
-    {
-      mLastRectangleUsedByRefreshContrastEnhancementIfNeeded = extent;
-
-      setContrastEnhancement( ce->contrastEnhancementAlgorithm(),
-                              rasterRenderer->minMaxOrigin().limits(),
-                              extent,
-                              static_cast<int>( SAMPLE_SIZE ),
-                              true,
-                              rasterRenderer );
-
-      // Update main renderer so that the legends get updated
-      if ( singleBandRenderer )
-        static_cast<QgsSingleBandGrayRenderer *>( renderer() )->setContrastEnhancement( new QgsContrastEnhancement( * singleBandRenderer->contrastEnhancement() ) );
-      else if ( multiBandRenderer )
-      {
-        if ( multiBandRenderer->redContrastEnhancement() )
-        {
-          static_cast<QgsMultiBandColorRenderer *>( renderer() )->setRedContrastEnhancement( new QgsContrastEnhancement( *multiBandRenderer->redContrastEnhancement() ) );
-        }
-        if ( multiBandRenderer->greenContrastEnhancement() )
-        {
-          static_cast<QgsMultiBandColorRenderer *>( renderer() )->setGreenContrastEnhancement( new QgsContrastEnhancement( *multiBandRenderer->greenContrastEnhancement() ) );
-        }
-        if ( multiBandRenderer->blueContrastEnhancement() )
-        {
-          static_cast<QgsMultiBandColorRenderer *>( renderer() )->setBlueContrastEnhancement( new QgsContrastEnhancement( *multiBandRenderer->blueContrastEnhancement() ) );
-        }
-      }
-
-      emitStyleChanged();
-      emit rendererChanged();
-    }
-  }
-}
-
 QString QgsRasterLayer::subsetString() const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
@@ -1694,7 +1585,21 @@ bool QgsRasterLayer::setSubsetString( const QString &subset )
   if ( res )
   {
     setExtent( mDataProvider->extent() );
-    refreshRenderer( renderer(), extent() );
+    QList<double> minValues;
+    QList<double> maxValues;
+    const QgsRasterMinMaxOrigin &minMaxOrigin = renderer()->minMaxOrigin();
+    for ( const int bandIdx : renderer()->usesBands() )
+    {
+      double min;
+      double max;
+
+      computeMinMax( bandIdx, minMaxOrigin, minMaxOrigin.limits(),
+                     extent(), static_cast<int>( QgsRasterLayer::SAMPLE_SIZE ),
+                     min, max );
+      minValues.append( min );
+      maxValues.append( max );
+    }
+    renderer()->refresh( extent(), minValues, maxValues, true );
     emit subsetStringChanged();
   }
 
@@ -1703,7 +1608,7 @@ bool QgsRasterLayer::setSubsetString( const QString &subset )
 
 bool QgsRasterLayer::defaultContrastEnhancementSettings(
   QgsContrastEnhancement::ContrastEnhancementAlgorithm &myAlgorithm,
-  QgsRasterMinMaxOrigin::Limits &myLimits ) const
+  Qgis::RasterRangeLimit &myLimits ) const
 {
   QGIS_PROTECT_QOBJECT_THREAD_ACCESS
 
@@ -1770,7 +1675,7 @@ void QgsRasterLayer::setDefaultContrastEnhancement()
   QgsDebugMsgLevel( QStringLiteral( "Entered" ), 4 );
 
   QgsContrastEnhancement::ContrastEnhancementAlgorithm myAlgorithm;
-  QgsRasterMinMaxOrigin::Limits myLimits;
+  Qgis::RasterRangeLimit myLimits;
   defaultContrastEnhancementSettings( myAlgorithm, myLimits );
 
   setContrastEnhancement( myAlgorithm, myLimits );
@@ -1822,6 +1727,44 @@ bool QgsRasterLayer::accept( QgsStyleEntityVisitorInterface *visitor ) const
   return true;
 }
 
+bool QgsRasterLayer::labelsEnabled() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mLabelsEnabled && static_cast< bool >( mLabeling );
+}
+
+void QgsRasterLayer::setLabelsEnabled( bool enabled )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  mLabelsEnabled = enabled;
+}
+
+const QgsAbstractRasterLayerLabeling *QgsRasterLayer::labeling() const
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mLabeling.get();
+}
+
+QgsAbstractRasterLayerLabeling *QgsRasterLayer::labeling()
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  return mLabeling.get();
+}
+
+void QgsRasterLayer::setLabeling( QgsAbstractRasterLayerLabeling *labeling )
+{
+  QGIS_PROTECT_QOBJECT_THREAD_ACCESS
+
+  if ( mLabeling.get() == labeling )
+    return;
+
+  mLabeling.reset( labeling );
+  triggerRepaint();
+}
 
 bool QgsRasterLayer::writeSld( QDomNode &node, QDomDocument &doc, QString &errorMessage, const QVariantMap &props ) const
 {
@@ -2290,6 +2233,20 @@ bool QgsRasterLayer::readSymbology( const QDomNode &layer_node, QString &errorMe
       mPipe->dataDefinedProperties().readXml( elemDataDefinedProperties, QgsRasterPipe::propertyDefinitions() );
   }
 
+  // read labeling definition
+  if ( categories.testFlag( Labeling ) )
+  {
+    QgsReadWriteContextCategoryPopper p = context.enterCategory( tr( "Labeling" ) );
+
+    QDomElement labelingElement = layer_node.firstChildElement( QStringLiteral( "labeling" ) );
+    if ( !labelingElement.isNull() )
+    {
+      QgsAbstractRasterLayerLabeling *labeling = QgsAbstractRasterLayerLabeling::createFromElement( labelingElement, context );
+      mLabelsEnabled = layer_node.toElement().attribute( QStringLiteral( "labelsEnabled" ), QStringLiteral( "0" ) ).toInt();
+      setLabeling( labeling );
+    }
+  }
+
   if ( categories.testFlag( MapTips ) )
   {
     QDomElement mapTipElem = layer_node.namedItem( QStringLiteral( "mapTip" ) ).toElement();
@@ -2497,6 +2454,17 @@ bool QgsRasterLayer::writeSymbology( QDomNode &layer_node, QDomDocument &documen
 
   writeCommonStyle( layerElement, document, context, categories );
 
+  if ( categories.testFlag( Labeling ) )
+  {
+    if ( mLabeling )
+    {
+      QDomElement labelingElement = mLabeling->save( document, context );
+      layerElement.appendChild( labelingElement );
+    }
+    if ( mLabelsEnabled )
+      layerElement.setAttribute( QStringLiteral( "labelsEnabled" ), QStringLiteral( "1" ) );
+  }
+
   // save map tip
   if ( categories.testFlag( MapTips ) )
   {
@@ -2697,3 +2665,5 @@ bool QgsRasterLayer::update()
   }
   return isValid();
 }
+
+#undef ERR
